@@ -1,4 +1,7 @@
 {-# LANGUAGE CPP #-}
+#ifndef NO_SAFE_HASKELL
+{-# LANGUAGE Safe #-}
+#endif
 module Test.QuickCheck
   (
     -- * Running tests
@@ -13,27 +16,39 @@ module Test.QuickCheck
   , verboseCheckWith
   , verboseCheckWithResult
   , verboseCheckResult
-  , verbose
+#ifndef NO_TEMPLATE_HASKELL
+    -- ** Testing all properties in a module
+  , quickCheckAll
+  , verboseCheckAll
+  , forAllProperties
+    -- ** Testing polymorphic properties
+  , polyQuickCheck
+  , polyVerboseCheck
+  , monomorphic
+#endif
 
     -- * Random generation
   , Gen
     -- ** Generator combinators
-  , sized
-  , resize
   , choose
-  , promote
-  , suchThat
-  , suchThatMaybe
   , oneof
   , frequency
   , elements
   , growingElements
+  , sized
+  , resize
+  , suchThat
+  , suchThatMaybe
   , listOf
   , listOf1
   , vectorOf
+  , infiniteListOf
     -- ** Generators which use Arbitrary
   , vector
   , orderedList
+  , infiniteList
+    -- ** Running a generator
+  , generate
     -- ** Generator debugging
   , sample
   , sample'
@@ -49,17 +64,24 @@ module Test.QuickCheck
   , arbitraryBoundedIntegral
   , arbitraryBoundedRandom
   , arbitraryBoundedEnum
-  , coarbitraryEnum
     -- ** Helper functions for implementing shrink
+#ifndef NO_GENERICS
+  , genericShrink
+  , subterms
+  , recursivelyShrink
+#endif
   , shrinkNothing
+  , shrinkList
   , shrinkIntegral
   , shrinkRealFrac
+  , shrinkRealFracToInteger
     -- ** Helper functions for implementing coarbitrary
   , variant
-  , (><)
   , coarbitraryIntegral
   , coarbitraryReal
   , coarbitraryShow
+  , coarbitraryEnum
+  , (><)
 
     -- ** Type-level modifiers for changing generator behavior
   , Blind(..)
@@ -69,44 +91,50 @@ module Test.QuickCheck
   , Positive(..)
   , NonZero(..)
   , NonNegative(..)
+  , Large(..)
+  , Small(..)
   , Smart(..)
   , Shrink2(..)
 #ifndef NO_MULTI_PARAM_TYPE_CLASSES
   , Shrinking(..)
-#endif
   , ShrinkState(..)
+#endif
 
     -- * Properties
-  , Property, Prop, Testable(..)
+  , Property, Testable(..)
     -- ** Property combinators
-  , mapSize
-  , shrinking
-  , (==>)
-  , discard
   , forAll
   , forAllShrink
-    -- *** Experimental combinators for conjunction and disjunction
+  , shrinking
+  , (==>)
+  , (===)
+  , ioProperty
+    -- *** Controlling property execution
+  , verbose
+  , once
+  , within
+  , noShrinking
+    -- *** Conjunction and disjunction
   , (.&.)
   , (.&&.)
   , conjoin
   , (.||.)
   , disjoin
-    -- *** Handling failure
-  , whenFail
+    -- *** What to do on failure
+  , counterexample
   , printTestCase
+  , whenFail
   , whenFail'
   , expectFailure
-  , within
-    -- *** Test distribution
+    -- *** Analysing test distribution
   , label
   , collect
   , classify
   , cover
-  , once
-
-    -- * Text formatting
-  , Str(..)
-  , ranges
+    -- *** Miscellaneous
+  , Discard(..)
+  , discard
+  , mapSize
   )
  where
 
@@ -120,6 +148,9 @@ import Test.QuickCheck.Property hiding ( Result(..) )
 import Test.QuickCheck.Test
 import Test.QuickCheck.Text
 import Test.QuickCheck.Exception
+#ifndef NO_TEMPLATE_HASKELL
+import Test.QuickCheck.All
+#endif
 
 --------------------------------------------------------------------------
 -- the end.
