@@ -1,3 +1,8 @@
+-- | Type classes for random generation of values.
+{-# LANGUAGE CPP #-}
+#ifndef NO_GENERICS
+{-# LANGUAGE DefaultSignatures, FlexibleContexts, TypeOperators #-}
+#endif
 module Test.QuickCheck.Arbitrary
   (
   -- * Arbitrary and CoArbitrary classes
@@ -5,34 +10,43 @@ module Test.QuickCheck.Arbitrary
   , CoArbitrary(..)
 
   -- ** Helper functions for implementing arbitrary
-  , arbitrarySizedIntegral        -- :: Num a => Gen a
+  , arbitrarySizedIntegral        -- :: Integral a => Gen a
   , arbitraryBoundedIntegral      -- :: (Bounded a, Integral a) => Gen a
   , arbitrarySizedBoundedIntegral -- :: (Bounded a, Integral a) => Gen a
   , arbitrarySizedFractional      -- :: Fractional a => Gen a
   , arbitraryBoundedRandom        -- :: (Bounded a, Random a) => Gen a
   , arbitraryBoundedEnum          -- :: (Bounded a, Enum a) => Gen a
   -- ** Helper functions for implementing shrink
+#ifndef NO_GENERICS
+  , genericShrink      -- :: (Generic a, Typeable a, RecursivelyShrink (Rep a), Subterms (Rep a)) => a -> [a]
+  , subterms           -- :: (Generic a, Subterms (Rep a)) => a -> [a]
+  , recursivelyShrink  -- :: (Generic a, RecursivelyShrink (Rep a)) => a -> [a]
+#endif
   , shrinkNothing            -- :: a -> [a]
   , shrinkList               -- :: (a -> [a]) -> [a] -> [[a]]
   , shrinkIntegral           -- :: Integral a => a -> [a]
   , shrinkRealFrac           -- :: RealFrac a => a -> [a]
+  , shrinkRealFracToInteger  -- :: RealFrac a => a -> [a]
   -- ** Helper functions for implementing coarbitrary
-  , (><)
   , coarbitraryIntegral      -- :: Integral a => a -> Gen b -> Gen b
   , coarbitraryReal          -- :: Real a => a -> Gen b -> Gen b
   , coarbitraryShow          -- :: Show a => a -> Gen b -> Gen b
   , coarbitraryEnum          -- :: Enum a => a -> Gen b -> Gen b
+  , (><)
 
   -- ** Generators which use arbitrary
-  , vector      -- :: Arbitrary a => Int -> Gen [a]
-  , orderedList -- :: (Ord a, Arbitrary a) => Gen [a]
+  , vector       -- :: Arbitrary a => Int -> Gen [a]
+  , orderedList  -- :: (Ord a, Arbitrary a) => Gen [a]
+  , infiniteList -- :: Arbitrary a => Gen [a]
   )
  where
 
 --------------------------------------------------------------------------
 -- imports
 
+import System.Random(Random)
 import Test.QuickCheck.Gen
+import Test.QuickCheck.Gen.Unsafe
 
 {-
 import Data.Generics
@@ -52,10 +66,12 @@ import Data.Char
   , isSpace
   )
 
+#ifndef NO_FIXED
 import Data.Fixed
   ( Fixed
   , HasResolution
   )
+#endif
 
 import Data.Ratio
   ( Ratio
@@ -67,10 +83,6 @@ import Data.Ratio
 import Data.Complex
   ( Complex((:+)) )
 
-import System.Random
-  ( Random
-  )
-
 import Data.List
   ( sort
   , nub
@@ -87,6 +99,11 @@ import Control.Monad
 import Data.Int(Int8, Int16, Int32, Int64)
 import Data.Word(Word, Word8, Word16, Word32, Word64)
 
+#ifndef NO_GENERICS
+import GHC.Generics
+import Data.Typeable
+#endif
+
 --------------------------------------------------------------------------
 -- ** class Arbitrary
 
@@ -97,10 +114,133 @@ class Arbitrary a where
   arbitrary = error "no default generator"
 
   -- | Produces a (possibly) empty list of all the possible
-  -- immediate shrinks of the given value.
+  -- immediate shrinks of the given value. The default implementation
+  -- returns the empty list, so will not try to shrink the value.
+  --
+  -- Most implementations of 'shrink' should try at least three things:
+  --
+  -- 1. Shrink a term to any of its immediate subterms.
+  --
+  -- 2. Recursively apply 'shrink' to all immediate subterms.
+  --
+  -- 3. Type-specific shrinkings such as replacing a constructor by a
+  --    simpler constructor.
+  --
+  -- For example, suppose we have the following implementation of binary trees:
+  --
+  -- > data Tree a = Nil | Branch a (Tree a) (Tree a)
+  --
+  -- We can then define 'shrink' as follows:
+  --
+  -- > shrink Nil = []
+  -- > shrink (Branch x l r) =
+  -- >   -- shrink Branch to Nil
+  -- >   [Nil] ++
+  -- >   -- shrink to subterms
+  -- >   [l, r] ++
+  -- >   -- recursively shrink subterms
+  -- >   [Branch x' l' r' | (x', l', r') <- shrink (x, l, r)]
+  --
+  -- There are a couple of subtleties here:
+  --
+  -- * QuickCheck tries the shrinking candidates in the order they
+  --   appear in the list, so we put more aggressive shrinking steps
+  --   (such as replacing the whole tree by @Nil@) before smaller
+  --   ones (such as recursively shrinking the subtrees).
+  --
+  -- * It is tempting to write the last line as
+  --   @[Branch x' l' r' | x' <- shrink x, l' <- shrink l, r' <- shrink r]@
+  --   but this is the /wrong thing/! It will force QuickCheck to shrink
+  --   @x@, @l@ and @r@ in tandem, and shrinking will stop once /one/ of
+  --   the three is fully shrunk.
+  --
+  -- There is a fair bit of boilerplate in the code above.
+  -- We can avoid it with the help of some generic functions;
+  -- note that these only work on GHC 7.2 and above.
+  -- The function 'genericShrink' tries shrinking a term to all of its
+  -- subterms and, failing that, recursively shrinks the subterms.
+  -- Using it, we can define 'shrink' as:
+  --
+  -- > shrink x = shrinkToNil x ++ genericShrink x
+  -- >   where
+  -- >     shrinkToNil Nil = []
+  -- >     shrinkToNil (Branch _ l r) = [Nil]
+  --
+  -- 'genericShrink' is a combination of 'subterms', which shrinks
+  -- a term to any of its subterms, and 'recursivelyShrink', which shrinks
+  -- all subterms of a term. These may be useful if you need a bit more
+  -- control over shrinking than 'genericShrink' gives you.
+  --
+  -- A final gotcha: we cannot define 'shrink' as simply @'shrink' x = Nil:'genericShrink' x@
+  -- as this shrinks @Nil@ to @Nil@, and shrinking will go into an
+  -- infinite loop.
+  --
+  -- If all this leaves you bewildered, you might try @'shrink' = 'genericShrink'@ to begin with,
+  -- after deriving @Generic@ and @Typeable@ for your type. However, if your data type has any
+  -- special invariants, you will need to check that 'genericShrink' can't break those invariants.
   shrink :: a -> [a]
   shrink _ = []
 
+#ifndef NO_GENERICS
+-- | Shrink a term to any of its immediate subterms,
+-- and also recursively shrink all subterms.
+genericShrink :: (Generic a, Typeable a, RecursivelyShrink (Rep a), Subterms (Rep a)) => a -> [a]
+genericShrink x = subterms x ++ recursivelyShrink x
+
+-- | Recursively shrink all immediate subterms.
+recursivelyShrink :: (Generic a, RecursivelyShrink (Rep a)) => a -> [a]
+recursivelyShrink = map to . grecursivelyShrink . from
+
+class RecursivelyShrink f where
+  grecursivelyShrink :: f a -> [f a]
+
+instance (RecursivelyShrink f, RecursivelyShrink g) => RecursivelyShrink (f :*: g) where
+  grecursivelyShrink (x :*: y) =
+    [x' :*: y | x' <- grecursivelyShrink x] ++
+    [x :*: y' | y' <- grecursivelyShrink y]
+
+instance (RecursivelyShrink f, RecursivelyShrink g) => RecursivelyShrink (f :+: g) where
+  grecursivelyShrink (L1 x) = map L1 (grecursivelyShrink x)
+  grecursivelyShrink (R1 x) = map R1 (grecursivelyShrink x)
+
+instance RecursivelyShrink f => RecursivelyShrink (M1 i c f) where
+  grecursivelyShrink (M1 x) = map M1 (grecursivelyShrink x)
+
+instance Arbitrary a => RecursivelyShrink (K1 i a) where
+  grecursivelyShrink (K1 x) = map K1 (shrink x)
+
+instance RecursivelyShrink U1 where
+  grecursivelyShrink U1 = []
+
+-- | All immediate subterms of a term.
+subterms :: (Generic a, Typeable a, Subterms (Rep a)) => a -> [a]
+subterms = gsubterms . from
+
+class Subterms f where
+  gsubterms :: Typeable b => f a -> [b]
+
+instance (Subterms f, Subterms g) => Subterms (f :*: g) where
+  gsubterms (x :*: y) =
+    gsubterms x ++ gsubterms y
+
+instance (Subterms f, Subterms g) => Subterms (f :+: g) where
+  gsubterms (L1 x) = gsubterms x
+  gsubterms (R1 x) = gsubterms x
+
+instance Subterms f => Subterms (M1 i c f) where
+  gsubterms (M1 x) = gsubterms x
+
+instance Typeable a => Subterms (K1 i a) where
+  gsubterms (K1 x) =
+    case cast x of
+      Nothing -> []
+      Just y -> [y]
+
+instance Subterms U1 where
+  gsubterms U1 = []
+
+#endif
+
 -- instances
 
 instance (CoArbitrary a, Arbitrary b) => Arbitrary (a -> b) where
@@ -115,7 +255,7 @@ instance Arbitrary Bool where
   shrink False = []
 
 instance Arbitrary Ordering where
-  arbitrary = arbitraryBoundedEnum
+  arbitrary = elements [LT, EQ, GT]
   shrink GT = [EQ, LT]
   shrink LT = [EQ]
   shrink EQ = []
@@ -139,6 +279,7 @@ instance Arbitrary a => Arbitrary [a] where
 
   shrink xs = shrinkList shrink xs
 
+-- | Shrink a list of values given a shrinking function for individual values.
 shrinkList :: (a -> [a]) -> [a] -> [[a]]
 shrinkList shr xs = concat [ removes k n xs | k <- takeWhile (>0) (iterate (`div`2) n) ]
                  ++ shrinkOne xs
@@ -167,54 +308,54 @@ shrinkList shr xs = concat [ removes k n xs | k <- takeWhile (>0) (iterate (`div
 
 instance (Integral a, Arbitrary a) => Arbitrary (Ratio a) where
   arbitrary = arbitrarySizedFractional
-  shrink    = shrinkRealFrac
+  shrink    = shrinkRealFracToInteger
 
 instance (RealFloat a, Arbitrary a) => Arbitrary (Complex a) where
   arbitrary = liftM2 (:+) arbitrary arbitrary
   shrink (x :+ y) = [ x' :+ y | x' <- shrink x ] ++
                     [ x :+ y' | y' <- shrink y ]
 
+#ifndef NO_FIXED
 instance HasResolution a => Arbitrary (Fixed a) where
-    arbitrary = arbitrarySizedFractional
-    shrink    = shrinkRealFrac
+  arbitrary = arbitrarySizedFractional
+  shrink    = shrinkRealFrac
+#endif
 
 instance (Arbitrary a, Arbitrary b)
       => Arbitrary (a,b)
  where
   arbitrary = liftM2 (,) arbitrary arbitrary
 
-  shrink (x,y) = [ (x',y) | x' <- shrink x ]
-              ++ [ (x,y') | y' <- shrink y ]
+  shrink (x, y) =
+       [ (x', y) | x' <- shrink x ]
+    ++ [ (x, y') | y' <- shrink y ]
 
 instance (Arbitrary a, Arbitrary b, Arbitrary c)
       => Arbitrary (a,b,c)
  where
   arbitrary = liftM3 (,,) arbitrary arbitrary arbitrary
 
-  shrink (x,y,z) = [ (x',y,z) | x' <- shrink x ]
-                ++ [ (x,y',z) | y' <- shrink y ]
-                ++ [ (x,y,z') | z' <- shrink z ]
+  shrink (x, y, z) =
+    [ (x', y', z')
+    | (x', (y', z')) <- shrink (x, (y, z)) ]
 
 instance (Arbitrary a, Arbitrary b, Arbitrary c, Arbitrary d)
       => Arbitrary (a,b,c,d)
  where
   arbitrary = liftM4 (,,,) arbitrary arbitrary arbitrary arbitrary
 
-  shrink (w,x,y,z) = [ (w',x,y,z) | w' <- shrink w ]
-                  ++ [ (w,x',y,z) | x' <- shrink x ]
-                  ++ [ (w,x,y',z) | y' <- shrink y ]
-                  ++ [ (w,x,y,z') | z' <- shrink z ]
+  shrink (w, x, y, z) =
+    [ (w', x', y', z')
+    | (w', (x', (y', z'))) <- shrink (w, (x, (y, z))) ]
 
 instance (Arbitrary a, Arbitrary b, Arbitrary c, Arbitrary d, Arbitrary e)
       => Arbitrary (a,b,c,d,e)
  where
   arbitrary = liftM5 (,,,,) arbitrary arbitrary arbitrary arbitrary arbitrary
 
-  shrink (v,w,x,y,z) = [ (v',w,x,y,z) | v' <- shrink v ]
-                    ++ [ (v,w',x,y,z) | w' <- shrink w ]
-                    ++ [ (v,w,x',y,z) | x' <- shrink x ]
-                    ++ [ (v,w,x,y',z) | y' <- shrink y ]
-                    ++ [ (v,w,x,y,z') | z' <- shrink z ]
+  shrink (v, w, x, y, z) =
+    [ (v', w', x', y', z')
+    | (v', (w', (x', (y', z')))) <- shrink (v, (w, (x, (y, z)))) ]
 
 -- typical instance for primitive (numerical) types
 
@@ -223,7 +364,7 @@ instance Arbitrary Integer where
   shrink    = shrinkIntegral
 
 instance Arbitrary Int where
-  arbitrary = arbitrarySizedBoundedIntegral
+  arbitrary = arbitrarySizedIntegral
   shrink    = shrinkIntegral
 
 instance Arbitrary Int8 where
@@ -292,11 +433,13 @@ instance Arbitrary Double where
 
 -- | Generates an integral number. The number can be positive or negative
 -- and its maximum absolute value depends on the size parameter.
-arbitrarySizedIntegral :: Num a => Gen a
+arbitrarySizedIntegral :: Integral a => Gen a
 arbitrarySizedIntegral =
   sized $ \n ->
-    let n' = toInteger n in
-      fmap fromInteger (choose (-n', n'))
+  inBounds fromInteger (choose (-toInteger n, toInteger n))
+
+inBounds :: Integral a => (Integer -> a) -> Gen Integer -> Gen a
+inBounds fi g = fmap fi (g `suchThat` (\x -> toInteger (fi x) == x))
 
 -- | Generates a fractional number. The number can be positive or negative
 -- and its maximum absolute value depends on the size parameter.
@@ -310,15 +453,19 @@ arbitrarySizedFractional =
  where
   precision = 9999999999999 :: Integer
 
+-- Useful for getting at minBound and maxBound without having to
+-- fiddle around with asTypeOf.
+withBounds :: Bounded a => (a -> a -> Gen a) -> Gen a
+withBounds k = k minBound maxBound
+
 -- | Generates an integral number. The number is chosen uniformly from
 -- the entire range of the type. You may want to use
 -- 'arbitrarySizedBoundedIntegral' instead.
 arbitraryBoundedIntegral :: (Bounded a, Integral a) => Gen a
 arbitraryBoundedIntegral =
-  do let mn = minBound
-         mx = maxBound `asTypeOf` mn
-     n <- choose (toInteger mn, toInteger mx)
-     return (fromInteger n `asTypeOf` mn)
+  withBounds $ \mn mx ->
+  do n <- choose (toInteger mn, toInteger mx)
+     return (fromInteger n)
 
 -- | Generates an element of a bounded type. The element is
 -- chosen from the entire range of the type.
@@ -328,10 +475,9 @@ arbitraryBoundedRandom = choose (minBound,maxBound)
 -- | Generates an element of a bounded enumeration.
 arbitraryBoundedEnum :: (Bounded a, Enum a) => Gen a
 arbitraryBoundedEnum =
-  do let mn = minBound
-         mx = maxBound `asTypeOf` mn
-     n <- choose (fromEnum mn, fromEnum mx)
-     return (toEnum n `asTypeOf` mn)
+  withBounds $ \mn mx ->
+  do n <- choose (fromEnum mn, fromEnum mx)
+     return (toEnum n)
 
 -- | Generates an integral number from a bounded domain. The number is
 -- chosen from the entire range of the type, but small numbers are
@@ -339,14 +485,13 @@ arbitraryBoundedEnum =
 -- Phil Wadler.
 arbitrarySizedBoundedIntegral :: (Bounded a, Integral a) => Gen a
 arbitrarySizedBoundedIntegral =
+  withBounds $ \mn mx ->
   sized $ \s ->
-    do let mn = minBound
-           mx = maxBound `asTypeOf` mn
-           bits n | n `quot` 2 == 0 = 0
+    do let bits n | n `quot` 2 == 0 = 0
                   | otherwise = 1 + bits (n `quot` 2)
            k  = 2^(s*(bits mn `max` bits mx `max` 40) `div` 100)
        n <- choose (toInteger mn `max` (-k), toInteger mx `min` k)
-       return (fromInteger n `asTypeOf` mn)
+       return (fromInteger n)
 
 -- ** Helper functions for implementing shrink
 
@@ -372,17 +517,23 @@ shrinkIntegral x =
             (True,  False) -> a + b < 0
             (False, True)  -> a + b > 0
 
--- | Shrink a fraction.
-shrinkRealFrac :: RealFrac a => a -> [a]
-shrinkRealFrac x =
+-- | Shrink a fraction, but only shrink to integral values.
+shrinkRealFracToInteger :: RealFrac a => a -> [a]
+shrinkRealFracToInteger x =
   nub $
   [ -x
   | x < 0
   ] ++
-  [ x'
-  | x' <- [fromInteger (truncate x)]
-  , x' << x
-  ]
+  map fromInteger (shrinkIntegral (truncate x))
+
+-- | Shrink a fraction.
+shrinkRealFrac :: RealFrac a => a -> [a]
+shrinkRealFrac x =
+  nub $
+  shrinkRealFracToInteger x ++
+  [ x - x'
+  | x' <- take 20 (iterate (/ 2) x)
+  , (x - x') << x ]
  where
   a << b = abs a < abs b
 
@@ -391,31 +542,25 @@ shrinkRealFrac x =
 
 -- | Used for random generation of functions.
 class CoArbitrary a where
-  -- | Used to generate a function of type @a -> c@. The implementation
-  -- should use the first argument to perturb the random generator
-  -- given as the second argument. the returned generator
-  -- is then used to generate the function result.
-  -- You can often use 'variant' and '><' to implement
-  -- 'coarbitrary'.
-  coarbitrary :: a -> Gen c -> Gen c
+  -- | Used to generate a function of type @a -> b@.
+  -- The first argument is a value, the second a generator.
+  -- You should use 'variant' to perturb the random generator;
+  -- the goal is that different values for the first argument will
+  -- lead to different calls to 'variant'. An example will help:
+  --
+  -- @
+  -- instance CoArbitrary a => CoArbitrary [a] where
+  --   coarbitrary []     = 'variant' 0
+  --   coarbitrary (x:xs) = 'variant' 1 . coarbitrary (x,xs)
+  -- @
 
-{-
-  -- GHC definition:
-  coarbitrary{| Unit |}    Unit      = id
-  coarbitrary{| a :*: b |} (x :*: y) = coarbitrary x >< coarbitrary y
-  coarbitrary{| a :+: b |} (Inl x)   = variant 0    . coarbitrary x
-  coarbitrary{| a :+: b |} (Inr y)   = variant (-1) . coarbitrary y
--}
+  coarbitrary :: a -> Gen b -> Gen b
 
+{-# DEPRECATED (><) "Use ordinary function composition instead" #-}
 -- | Combine two generator perturbing functions, for example the
 -- results of calls to 'variant' or 'coarbitrary'.
 (><) :: (Gen a -> Gen a) -> (Gen a -> Gen a) -> (Gen a -> Gen a)
-(><) f g gen =
-  do n <- arbitrary
-     (g . variant (n :: Int) . f) gen
-
--- for the sake of non-GHC compilers, I have added definitions
--- for coarbitrary here.
+(><) = (.)
 
 instance (Arbitrary a, CoArbitrary b) => CoArbitrary (a -> b) where
   coarbitrary f gen =
@@ -427,63 +572,65 @@ instance CoArbitrary () where
 
 instance CoArbitrary Bool where
   coarbitrary False = variant 0
-  coarbitrary True  = variant (-1)
+  coarbitrary True  = variant 1
 
 instance CoArbitrary Ordering where
-  coarbitrary GT = variant 1
-  coarbitrary EQ = variant 0
-  coarbitrary LT = variant (-1)
+  coarbitrary GT = variant 0
+  coarbitrary EQ = variant 1
+  coarbitrary LT = variant 2
 
 instance CoArbitrary a => CoArbitrary (Maybe a) where
   coarbitrary Nothing  = variant 0
-  coarbitrary (Just x) = variant (-1) . coarbitrary x
+  coarbitrary (Just x) = variant 1 . coarbitrary x
 
 instance (CoArbitrary a, CoArbitrary b) => CoArbitrary (Either a b) where
-  coarbitrary (Left x)  = variant 0    . coarbitrary x
-  coarbitrary (Right y) = variant (-1) . coarbitrary y
+  coarbitrary (Left x)  = variant 0 . coarbitrary x
+  coarbitrary (Right y) = variant 1 . coarbitrary y
 
 instance CoArbitrary a => CoArbitrary [a] where
   coarbitrary []     = variant 0
-  coarbitrary (x:xs) = variant (-1) . coarbitrary (x,xs)
+  coarbitrary (x:xs) = variant 1 . coarbitrary (x,xs)
 
 instance (Integral a, CoArbitrary a) => CoArbitrary (Ratio a) where
   coarbitrary r = coarbitrary (numerator r,denominator r)
 
+#ifndef NO_FIXED
 instance HasResolution a => CoArbitrary (Fixed a) where
   coarbitrary = coarbitraryReal
+#endif
 
 instance (RealFloat a, CoArbitrary a) => CoArbitrary (Complex a) where
-  coarbitrary (x :+ y) = coarbitrary x >< coarbitrary y
+  coarbitrary (x :+ y) = coarbitrary x . coarbitrary y
 
 instance (CoArbitrary a, CoArbitrary b)
       => CoArbitrary (a,b)
  where
   coarbitrary (x,y) = coarbitrary x
-                   >< coarbitrary y
+                    . coarbitrary y
 
 instance (CoArbitrary a, CoArbitrary b, CoArbitrary c)
       => CoArbitrary (a,b,c)
  where
   coarbitrary (x,y,z) = coarbitrary x
-                     >< coarbitrary y
-                     >< coarbitrary z
+                      . coarbitrary y
+                      . coarbitrary z
 
 instance (CoArbitrary a, CoArbitrary b, CoArbitrary c, CoArbitrary d)
       => CoArbitrary (a,b,c,d)
  where
   coarbitrary (x,y,z,v) = coarbitrary x
-                       >< coarbitrary y
-                       >< coarbitrary z
-                       >< coarbitrary v
+                        . coarbitrary y
+                        . coarbitrary z
+                        . coarbitrary v
 
 instance (CoArbitrary a, CoArbitrary b, CoArbitrary c, CoArbitrary d, CoArbitrary e)
       => CoArbitrary (a,b,c,d,e)
  where
   coarbitrary (x,y,z,v,w) = coarbitrary x
-                         >< coarbitrary y
-                         >< coarbitrary z
-                         >< coarbitrary v
-                         >< coarbitrary w
+                          . coarbitrary y
+                          . coarbitrary z
+                          . coarbitrary v
+                          . coarbitrary w
 
 -- typical instance for primitive (numerical) types
 
@@ -560,5 +707,9 @@ vector k = vectorOf k arbitrary
 orderedList :: (Ord a, Arbitrary a) => Gen [a]
 orderedList = sort `fmap` arbitrary
 
+-- | Generate an infinite list.
+infiniteList :: Arbitrary a => Gen [a]
+infiniteList = infiniteListOf arbitrary
+
 --------------------------------------------------------------------------
 -- the end.
