@@ -1,6 +1,6 @@
 {-# LANGUAGE TypeOperators, GADTs #-}
 -- | Generation of random shrinkable, showable functions.
--- Not really documented at the moment!
+-- See the paper \"Shrinking and showing functions\" by Koen Claessen.
 --
 -- Example of use:
 --
@@ -277,8 +277,7 @@ instance (Function a, CoArbitrary a, Arbitrary b) => Arbitrary (Fun a b) where
        return (mkFun p d)
 
   shrink (Fun (p,d) _) =
-       [ mkFun p' d | p' <- shrink p ]
-    ++ [ mkFun p d' | d' <- shrink d ]
+    [ mkFun p' d' | (p', d') <- shrink (p, d) ]
 
 --------------------------------------------------------------------------
 -- the end.
