@@ -1,3 +1,7 @@
+{-# LANGUAGE CPP #-}
+#ifndef NO_ST_MONAD
+{-# LANGUAGE Rank2Types #-}
+#endif
 -- | Test case generation.
 module Test.QuickCheck.Gen where
 
@@ -21,10 +25,16 @@ import Control.Applicative
   ( Applicative(..)
   )
 
+import Test.QuickCheck.Random
+
 --------------------------------------------------------------------------
 -- ** Generator type
 
-newtype Gen a = MkGen{ unGen :: StdGen -> Int -> a }
+-- | A generator for values of type @a@.
+newtype Gen a = MkGen{
+  unGen :: QCGen -> Int -> a -- ^ Run the generator on a particular seed.
+                             -- If you just want to get a random value out, consider using 'generate'.
+  }
 
 instance Functor Gen where
   fmap f (MkGen h) =
@@ -50,13 +60,7 @@ instance Monad Gen where
 
 -- | Modifies a generator using an integer seed.
 variant :: Integral n => n -> Gen a -> Gen a
-variant k0 (MkGen m) = MkGen (\r n -> m (var k0 r) n)
- where
-  var k = (if k == k' then id  else var k')
-        . (if even k  then fst else snd)
-        . split
-   where
-    k' = k `div` 2
+variant k (MkGen g) = MkGen (\r n -> g (variantQCGen k r) n)
 
 -- | Used to construct generators that depend on the size parameter.
 sized :: (Int -> Gen a) -> Gen a
@@ -71,22 +75,22 @@ resize n (MkGen m) = MkGen (\r _ -> m r n)
 choose :: Random a => (a,a) -> Gen a
 choose rng = MkGen (\r _ -> let (x,_) = randomR rng r in x)
 
--- | Promotes a monadic generator to a generator of monadic values.
-promote :: Monad m => m (Gen a) -> Gen (m a)
-promote m = MkGen (\r n -> liftM (\(MkGen m') -> m' r n) m)
+-- | Run a generator.
+generate :: Gen a -> IO a
+generate (MkGen g) =
+  do r <- newQCGen
+     return (g r 30)
 
 -- | Generates some example values.
 sample' :: Gen a -> IO [a]
-sample' (MkGen m) =
-  do rnd0 <- newStdGen
-     let rnds rnd = rnd1 : rnds rnd2 where (rnd1,rnd2) = split rnd
-     return [(m r n) | (r,n) <- rnds rnd0 `zip` [0,2..20] ]
+sample' g =
+  generate (sequence [ resize n g | n <- [0,2..20] ])
 
 -- | Generates some example values and prints them to 'stdout'.
 sample :: Show a => Gen a -> IO ()
 sample g =
   do cases <- sample' g
-     sequence_ (map print cases)
+     mapM_ print cases
 
 --------------------------------------------------------------------------
 -- ** Common generator combinators
@@ -139,10 +143,11 @@ growingElements :: [a] -> Gen a
 growingElements [] = error "QuickCheck.growingElements used with empty list"
 growingElements xs = sized $ \n -> elements (take (1 `max` size n) xs)
   where
-   k      = length xs
-   mx     = 100
-   log'   = round . log . fromIntegral
-   size n = (log' n + 1) * k `div` log' mx
+   k        = length xs
+   mx       = 100
+   log'     = round . log . toDouble
+   size n   = (log' n + 1) * k `div` log' mx
+   toDouble = fromIntegral :: Int -> Double
 
 {- WAS:
 growingElements xs = sized $ \n -> elements (take (1 `max` (n * k `div` 100)) xs)
@@ -168,5 +173,9 @@ listOf1 gen = sized $ \n ->
 vectorOf :: Int -> Gen a -> Gen [a]
 vectorOf k gen = sequence [ gen | _ <- [1..k] ]
 
+-- | Generates an infinite list.
+infiniteListOf :: Gen a -> Gen [a]
+infiniteListOf gen = sequence (repeat gen)
+
 --------------------------------------------------------------------------
 -- the end.
