@@ -1,10 +1,15 @@
+-- | Combinators for constructing properties.
 {-# LANGUAGE CPP #-}
+#ifndef NO_SAFE_HASKELL
+{-# LANGUAGE Safe #-}
+#endif
 module Test.QuickCheck.Property where
 
 --------------------------------------------------------------------------
 -- imports
 
 import Test.QuickCheck.Gen
+import Test.QuickCheck.Gen.Unsafe
 import Test.QuickCheck.Arbitrary
 import Test.QuickCheck.Text( showErr, isOneLine, putLine )
 import Test.QuickCheck.Exception
@@ -14,6 +19,8 @@ import Test.QuickCheck.State
 import System.Timeout(timeout)
 #endif
 import Data.Maybe
+import Control.Applicative
+import Control.Monad
 
 --------------------------------------------------------------------------
 -- fixities
@@ -55,33 +62,67 @@ infixr 1 .||.
 --------------------------------------------------------------------------
 -- * Property and Testable types
 
-type Property = Gen Prop
+-- | The type of properties.
+--
+-- Backwards combatibility note: in older versions of QuickCheck
+-- 'Property' was a type synonym for @'Gen' 'Prop'@, so you could mix
+-- and match property combinators and 'Gen' monad operations. Code
+-- that does this will no longer typecheck.
+-- However, it is easy to fix: because of the 'Testable' typeclass, any
+-- combinator that expects a 'Property' will also accept a @'Gen' 'Property'@.
+-- If you have a 'Property' where you need a @'Gen' 'a'@, simply wrap
+-- the property combinator inside a 'return' to get a @'Gen' 'Property'@, and
+-- all should be well.
+newtype Property = MkProperty { unProperty :: Gen Prop }
 
 -- | The class of things which can be tested, i.e. turned into a property.
 class Testable prop where
+  -- | Convert the thing to a property.
   property :: prop -> Property
+  -- | If true, the property will only be tested once.
+  -- However, if used inside a quantifier, it will be tested normally.
   exhaustive :: prop -> Bool
   exhaustive _ = False
 
+-- | If a property returns 'Discard', the current test case is discarded,
+-- the same as if a precondition was false.
+data Discard = Discard
+
+instance Testable Discard where
+  property _ = property rejected
+  exhaustive _ = True
+
 instance Testable Bool where
   property = property . liftBool
   exhaustive _ = True
 
 instance Testable Result where
-  property = return . MkProp . protectResults . return
+  property = MkProperty . return . MkProp . protectResults . return
   exhaustive _ = True
 
 instance Testable Prop where
-  property (MkProp r) = return . MkProp . ioRose . return $ r
+  property (MkProp r) = MkProperty . return . MkProp . ioRose . return $ r
   exhaustive _ = True
 
 instance Testable prop => Testable (Gen prop) where
-  property mp = do p <- mp; property p
+  property mp = MkProperty $ do p <- mp; unProperty (property p)
+
+instance Testable Property where
+  property = property . unProperty
 
 -- | Do I/O inside a property. This can obviously lead to unrepeatable
 -- testcases, so use with care.
+{-# DEPRECATED morallyDubiousIOProperty "Use ioProperty instead" #-}
 morallyDubiousIOProperty :: Testable prop => IO prop -> Property
-morallyDubiousIOProperty = fmap (MkProp . ioRose . fmap unProp) . promote . fmap property
+morallyDubiousIOProperty = ioProperty -- Silly names aren't all they're cracked up to be :)
+
+-- | Do I/O inside a property. This can obviously lead to unrepeatable
+-- testcases, so use with care.
+--
+-- For more advanced monadic testing you may want to look at
+-- "Test.QuickCheck.Monadic".
+ioProperty :: Testable prop => IO prop -> Property
+ioProperty = MkProperty . fmap (MkProp . ioRose . fmap unProp) . promote . fmap (unProperty . property)
 
 instance (Arbitrary a, Show a, Testable prop) => Testable (a -> prop) where
   property f = forAllShrink arbitrary shrink f
@@ -117,6 +158,11 @@ instance Functor Rose where
   fmap f (IORose rs)   = IORose (fmap (fmap f) rs)
   fmap f (MkRose x rs) = MkRose (f x) [ fmap f r | r <- rs ]
 
+instance Applicative Rose where
+  pure = return
+  -- f must be total
+  (<*>) = liftM2 ($)
+
 instance Monad Rose where
   return x = MkRose x []
   -- k must be total
@@ -157,32 +203,32 @@ data CallbackKind = Counterexample    -- ^ Affected by the 'verbose' combinator
 -- | The result of a single test.
 data Result
   = MkResult
-  { ok          :: Maybe Bool     -- ^ result of the test case; Nothing = discard
-  , expect      :: Bool           -- ^ indicates what the expected result of the property is
-  , reason      :: String         -- ^ a message indicating what went wrong
-  , interrupted :: Bool           -- ^ indicates if the test case was cancelled by pressing ^C
-  , abort       :: Bool           -- ^ if True, the test should not be repeated
-  , stamp       :: [(String,Int)] -- ^ the collected values for this test case
-  , callbacks   :: [Callback]     -- ^ the callbacks for this test case
+  { ok           :: Maybe Bool        -- ^ result of the test case; Nothing = discard
+  , expect       :: Bool              -- ^ indicates what the expected result of the property is
+  , reason       :: String            -- ^ a message indicating what went wrong
+  , theException :: Maybe AnException -- ^ the exception thrown, if any
+  , abort        :: Bool              -- ^ if True, the test should not be repeated
+  , stamp        :: [(String,Int)]    -- ^ the collected values for this test case
+  , callbacks    :: [Callback]        -- ^ the callbacks for this test case
   }
 
 result :: Result
 result =
   MkResult
-  { ok          = undefined
-  , expect      = True
-  , reason      = ""
-  , interrupted = False
-  , abort       = False
-  , stamp       = []
-  , callbacks   = []
+  { ok           = undefined
+  , expect       = True
+  , reason       = ""
+  , theException = Nothing
+  , abort        = False
+  , stamp        = []
+  , callbacks    = []
   }
 
 exception :: String -> AnException -> Result
 exception msg err
   | isDiscard err = rejected
   | otherwise = failed{ reason = formatException msg err,
-                        interrupted = isInterrupt err }
+                        theException = Just err }
 
 formatException :: String -> AnException -> String
 formatException msg err = msg ++ ":" ++ format (show err)
@@ -219,14 +265,14 @@ mapRoseResult :: Testable prop => (Rose Result -> Rose Result) -> prop -> Proper
 mapRoseResult f = mapProp (\(MkProp t) -> MkProp (f t))
 
 mapProp :: Testable prop => (Prop -> Prop) -> prop -> Property
-mapProp f = fmap f . property
+mapProp f = MkProperty . fmap f . unProperty . property
 
 --------------------------------------------------------------------------
 -- ** Property combinators
 
 -- | Changes the maximum test case size for a property.
 mapSize :: Testable prop => (Int -> Int) -> prop -> Property
-mapSize f p = sized ((`resize` property p) . f)
+mapSize f p = MkProperty (sized ((`resize` unProperty (property p)) . f))
 
 -- | Shrinks the argument to property if it fails. Shrinking is done
 -- automatically for most types. This is only needed when you want to
@@ -235,10 +281,10 @@ shrinking :: Testable prop =>
              (a -> [a])  -- ^ 'shrink'-like function.
           -> a           -- ^ The original argument
           -> (a -> prop) -> Property
-shrinking shrinker x0 pf = fmap (MkProp . joinRose . fmap unProp) (promote (props x0))
+shrinking shrinker x0 pf = MkProperty (fmap (MkProp . joinRose . fmap unProp) (promote (props x0)))
  where
   props x =
-    MkRose (property (pf x)) [ props x' | x' <- shrinker x ]
+    MkRose (unProperty (property (pf x))) [ props x' | x' <- shrinker x ]
 
 -- | Disables shrinking for a property altogether.
 noShrinking :: Testable prop => prop -> Property
@@ -248,9 +294,9 @@ noShrinking = mapRoseResult (onRose (\res _ -> MkRose res []))
 callback :: Testable prop => Callback -> prop -> Property
 callback cb = mapTotalResult (\res -> res{ callbacks = cb : callbacks res })
 
--- | Prints a message to the terminal as part of the counterexample.
-printTestCase :: Testable prop => String -> prop -> Property
-printTestCase s =
+-- | Adds the given string to the counterexample.
+counterexample :: Testable prop => String -> prop -> Property
+counterexample s =
   callback $ PostFinalFailure Counterexample $ \st _res -> do
     res <- tryEvaluateIO (putLine (terminal st) s)
     case res of
@@ -259,6 +305,11 @@ printTestCase s =
       Right () ->
         return ()
 
+-- | Adds the given string to the counterexample.
+{-# DEPRECATED printTestCase "Use counterexample instead" #-}
+printTestCase :: Testable prop => String -> prop -> Property
+printTestCase = counterexample
+
 -- | Performs an 'IO' action after the last failure of a property.
 whenFail :: Testable prop => IO () -> prop -> Property
 whenFail m =
@@ -275,8 +326,7 @@ whenFail' m =
       then m
       else return ()
 
--- | Prints out the generated testcase every time the property is tested,
--- like 'verboseCheck' from QuickCheck 1.
+-- | Prints out the generated testcase every time the property is tested.
 -- Only variables quantified over /inside/ the 'verbose' are printed.
 verbose :: Testable prop => prop -> Property
 verbose = mapResult (\res -> res { callbacks = newCallbacks (callbacks res) ++ callbacks res })
@@ -287,7 +337,8 @@ verbose = mapResult (\res -> res { callbacks = newCallbacks (callbacks res) ++ c
         status MkResult{ok = Just False} = "Failed"
         status MkResult{ok = Nothing} = "Skipped (precondition false)"
 
--- | Modifies a property so that it is expected to fail for some test cases.
+-- | Indicates that a property is supposed to fail.
+-- QuickCheck will report an error if it does not fail.
 expectFailure :: Testable prop => prop -> Property
 expectFailure = mapTotalResult (\res -> res{ expect = False })
 
@@ -329,7 +380,7 @@ cover False _ _ = property
 -- the first argument is 'False' (in which case the test case is discarded),
 -- or if the given property holds.
 (==>) :: Testable prop => Bool -> prop -> Property
-False ==> _ = property rejected
+False ==> _ = property Discard
 True  ==> p = property p
 
 -- | Considers a property failed if it does not complete within
@@ -355,24 +406,29 @@ within n = mapRoseResult f
 forAll :: (Show a, Testable prop)
        => Gen a -> (a -> prop) -> Property
 forAll gen pf =
+  MkProperty $
   gen >>= \x ->
-    printTestCase (show x) (pf x)
+    unProperty (counterexample (show x) (pf x))
 
 -- | Like 'forAll', but tries to shrink the argument for failing test cases.
 forAllShrink :: (Show a, Testable prop)
              => Gen a -> (a -> [a]) -> (a -> prop) -> Property
 forAllShrink gen shrinker pf =
+  MkProperty $
   gen >>= \x ->
+    unProperty $
     shrinking shrinker x $ \x' ->
-      printTestCase (show x') (pf x')
+      counterexample (show x') (pf x')
 
 -- | Nondeterministic choice: 'p1' '.&.' 'p2' picks randomly one of
 -- 'p1' and 'p2' to test. If you test the property 100 times it
 -- makes 100 random choices.
 (.&.) :: (Testable prop1, Testable prop2) => prop1 -> prop2 -> Property
 p1 .&. p2 =
+  MkProperty $
   arbitrary >>= \b ->
-    printTestCase (if b then "LHS" else "RHS") $
+    unProperty $
+    counterexample (if b then "LHS" else "RHS") $
       if b then property p1 else property p2
 
 -- | Conjunction: 'p1' '.&&.' 'p2' passes if both 'p1' and 'p2' pass.
@@ -382,7 +438,8 @@ p1 .&&. p2 = conjoin [property p1, property p2]
 -- | Take the conjunction of several properties.
 conjoin :: Testable prop => [prop] -> Property
 conjoin ps =
-  do roses <- mapM (fmap unProp . property) ps
+  MkProperty $
+  do roses <- mapM (fmap unProp . unProperty . property) ps
      return (MkProp (conj [] roses))
  where
   conj cbs [] =
@@ -411,7 +468,8 @@ p1 .||. p2 = disjoin [property p1, property p2]
 -- | Take the disjunction of several properties.
 disjoin :: Testable prop => [prop] -> Property
 disjoin ps =
-  do roses <- mapM (fmap unProp . property) ps
+  MkProperty $
+  do roses <- mapM (fmap unProp . unProperty . property) ps
      return (MkProp (foldr disj (MkRose failed []) roses))
  where
   disj :: Rose Result -> Rose Result -> Rose Result
@@ -440,13 +498,20 @@ disjoin ps =
   result1 >>> result2 | not (expect result1 && expect result2) = expectFailureError
   result1 >>> result2 =
     result2
-    { reason      = if null (reason result2) then reason result1 else reason result2
-    , interrupted = interrupted result1 || interrupted result2
-    , stamp       = stamp result1 ++ stamp result2
-    , callbacks   = callbacks result1 ++
+    { reason       = if null (reason result2) then reason result1 else reason result2
+    , theException = if null (reason result2) then theException result1 else theException result2
+    , stamp        = stamp result1 ++ stamp result2
+    , callbacks    = callbacks result1 ++
                     [PostFinalFailure Counterexample $ \st _res -> putLine (terminal st) ""] ++
                     callbacks result2
     }
 
+-- | Like '==', but prints a counterexample when it fails.
+infix 4 ===
+(===) :: (Eq a, Show a) => a -> a -> Property
+x === y =
+  counterexample (show x ++ " /= " ++ show y) (x == y)
+
+
 --------------------------------------------------------------------------
 -- the end.
